@@ -18,7 +18,7 @@ from fmpy.gui.generated.MainWindow import Ui_MainWindow
 import fmpy
 from fmpy import read_model_description, supported_platforms, platform
 from fmpy.model_description import ScalarVariable
-
+from fmpy.util import can_simulate
 
 from fmpy.gui.model import VariablesTableModel, VariablesTreeModel, VariablesModel, VariablesFilterModel
 from fmpy.gui.log import Log, LogMessagesFilterProxyModel
@@ -31,7 +31,7 @@ import pyqtgraph as pg
 
 pg.setConfigOptions(background='w', foreground='k', antialias=True)
 
-COLLAPSABLE_COLUMNS = ['Value Reference', 'Initial', 'Causality', 'Variability', 'Min', 'Max']
+COLLAPSABLE_COLUMNS = ['Type', 'Value Reference', 'Initial', 'Causality', 'Variability', 'Min', 'Max']
 
 
 class ClickableLabel(QLabel):
@@ -104,6 +104,8 @@ class MainWindow(QMainWindow):
         self.ui = Ui_MainWindow()
         self.ui.setupUi(self)
 
+        self.showColumnActions = {}
+
         # use a smaller default font size on Mac and Linux
         if sys.platform in ['darwin', 'linux']:
             defaultFont = QFont()
@@ -119,7 +121,7 @@ class MainWindow(QMainWindow):
         geo = QApplication.desktop().availableGeometry()
         width = min(geo.width() * 0.85, 1100.0)
         height = min(geo.height() * 0.85, 900.0)
-        self.resize(width, height)
+        self.resize(int(width), int(height))
 
         # hide the variables
         self.ui.dockWidget.hide()
@@ -240,11 +242,14 @@ class MainWindow(QMainWindow):
         self.columnsMenu = self.contextMenu.addMenu('Columns')
         action = self.columnsMenu.addAction('Show All')
         action.triggered.connect(self.showAllColumns)
+        action = self.columnsMenu.addAction('Hide All')
+        action.triggered.connect(self.hideAllColumns)
         self.columnsMenu.addSeparator()
         for column in COLLAPSABLE_COLUMNS:
             action = self.columnsMenu.addAction(column)
             action.setCheckable(True)
             action.toggled.connect(lambda show, col=column: self.showColumn(col, show))
+            self.showColumnActions[column] = action
         self.contextMenu.addSeparator()
         self.actionClearPlots = self.contextMenu.addAction("Clear Plots", self.clearPlots)
 
@@ -256,16 +261,20 @@ class MainWindow(QMainWindow):
 
         # tools menu
         self.ui.actionValidateFMU.triggered.connect(self.validateFMU)
-        self.ui.actionCompilePlatformBinary.triggered.connect(self.compilePlatformBinary)
+        self.ui.actionCompileDarwinBinary.triggered.connect(lambda: self.compilePlatformBinary('darwin64'))
+        self.ui.actionCompileLinuxBinary.triggered.connect(lambda: self.compilePlatformBinary('linux64'))
+        self.ui.actionCompileWin32Binary.triggered.connect(lambda: self.compilePlatformBinary('win32'))
+        self.ui.actionCompileWin64Binary.triggered.connect(lambda: self.compilePlatformBinary('win64'))
         self.ui.actionCreateJupyterNotebook.triggered.connect(self.createJupyterNotebook)
         self.ui.actionCreateCMakeProject.triggered.connect(self.createCMakeProject)
-        self.ui.actionAddRemoting.triggered.connect(self.addRemoting)
+        self.ui.actionAddWindows32Remoting.triggered.connect(lambda: self.addRemotingBinaries('win64', 'win32'))
+        self.ui.actionAddLinux64Remoting.triggered.connect(lambda: self.addRemotingBinaries('linux64', 'win64'))
         self.ui.actionAddCoSimulationWrapper.triggered.connect(self.addCoSimulationWrapper)
 
         # help menu
-        self.ui.actionOpenFMI1SpecCS.triggered.connect(lambda: QDesktopServices.openUrl(QUrl('https://svn.modelica.org/fmi/branches/public/specifications/v1.0/FMI_for_CoSimulation_v1.0.1.pdf')))
-        self.ui.actionOpenFMI1SpecME.triggered.connect(lambda: QDesktopServices.openUrl(QUrl('https://svn.modelica.org/fmi/branches/public/specifications/v1.0/FMI_for_ModelExchange_v1.0.1.pdf')))
-        self.ui.actionOpenFMI2Spec.triggered.connect(lambda: QDesktopServices.openUrl(QUrl('https://github.com/modelica/fmi-standard/releases/download/v2.0.2/FMI-Specification-2.0.2.pdf')))
+        self.ui.actionOpenFMI1SpecCS.triggered.connect(lambda: QDesktopServices.openUrl(QUrl('https://fmi-standard.org/assets/releases/FMI_for_CoSimulation_v1.0.1.pdf')))
+        self.ui.actionOpenFMI1SpecME.triggered.connect(lambda: QDesktopServices.openUrl(QUrl('https://fmi-standard.org/assets/releases/FMI_for_ModelExchange_v1.0.1.pdf')))
+        self.ui.actionOpenFMI2Spec.triggered.connect(lambda: QDesktopServices.openUrl(QUrl('https://github.com/modelica/fmi-standard/releases/download/v2.0.3/FMI-Specification-2.0.3.pdf')))
         self.ui.actionOpenTestFMUs.triggered.connect(lambda: QDesktopServices.openUrl(QUrl('https://github.com/modelica/fmi-cross-check/tree/master/fmus')))
         self.ui.actionOpenWebsite.triggered.connect(lambda: QDesktopServices.openUrl(QUrl('https://github.com/CATIA-Systems/FMPy')))
         self.ui.actionShowReleaseNotes.triggered.connect(lambda: QDesktopServices.openUrl(QUrl('https://fmpy.readthedocs.io/en/latest/changelog/')))
@@ -422,7 +431,7 @@ class MainWindow(QMainWindow):
 
         for v in md.modelVariables:
             self.variables[v.name] = v
-            if v.causality == 'output':
+            if v.causality == 'output' and not v.dimensions:
                 self.selectedVariables.add(v)
 
         fmi_types = []
@@ -441,13 +450,18 @@ class MainWindow(QMainWindow):
         self.ui.actionValidateFMU.setEnabled(True)
 
         can_compile = md.fmiVersion == '2.0' and 'c-code' in platforms
-        self.ui.actionCompilePlatformBinary.setEnabled(can_compile)
+
+        self.ui.actionCompileDarwinBinary.setEnabled(can_compile and fmpy.system == 'darwin')
+        self.ui.actionCompileLinuxBinary.setEnabled(can_compile and fmpy.system in ['linux', 'windows'])
+        self.ui.actionCompileWin32Binary.setEnabled(can_compile and fmpy.system == 'windows')
+        self.ui.actionCompileWin64Binary.setEnabled(can_compile and fmpy.system == 'windows')
+
         self.ui.actionCreateCMakeProject.setEnabled(can_compile)
 
         self.ui.actionCreateJupyterNotebook.setEnabled(True)
 
-        can_add_remoting = md.fmiVersion == '2.0' and 'win32' in platforms and 'win64' not in platforms
-        self.ui.actionAddRemoting.setEnabled(can_add_remoting)
+        self.ui.actionAddWindows32Remoting.setEnabled(md.fmiVersion == '2.0' and 'win32' in platforms and 'win64' not in platforms)
+        self.ui.actionAddLinux64Remoting.setEnabled(md.fmiVersion == '2.0' and 'win64' in platforms and 'linux64' not in platforms)
 
         can_add_cswrapper = md.fmiVersion == '2.0' and md.coSimulation is None and md.modelExchange is not None
         self.ui.actionAddCoSimulationWrapper.setEnabled(can_add_cswrapper)
@@ -504,13 +518,13 @@ class MainWindow(QMainWindow):
         self.ui.actionShowLog.setEnabled(True)
         self.ui.actionShowResults.setEnabled(False)
 
-        can_simulate = platform in platforms or platform == 'win64' and 'win32' in platforms
+        can_sim, _ = can_simulate(platforms)
 
-        self.ui.actionLoadStartValues.setEnabled(can_simulate)
-        self.ui.actionSimulate.setEnabled(can_simulate)
-        self.stopTimeLineEdit.setEnabled(can_simulate)
-        self.fmiTypeComboBox.setEnabled(can_simulate and len(fmi_types) > 1)
-        self.ui.settingsGroupBox.setEnabled(can_simulate)
+        self.ui.actionLoadStartValues.setEnabled(can_sim)
+        self.ui.actionSimulate.setEnabled(can_sim)
+        self.stopTimeLineEdit.setEnabled(can_sim)
+        self.fmiTypeComboBox.setEnabled(can_sim and len(fmi_types) > 1)
+        self.ui.settingsGroupBox.setEnabled(can_sim)
 
         settings = QSettings()
         recent_files = settings.value("recentFiles", defaultValue=[])
@@ -828,6 +842,8 @@ class MainWindow(QMainWindow):
         self.updatePlotData()
 
     def showColumn(self, name, show):
+        if name in self.showColumnActions:
+            self.showColumnActions[name].setChecked(show)
         i = VariablesModel.COLUMN_NAMES.index(name)
         self.ui.treeView.setColumnHidden(i, not show)
         self.ui.tableView.setColumnHidden(i, not show)
@@ -1048,14 +1064,16 @@ class MainWindow(QMainWindow):
         """ Create the graphical representation of the FMU's inputs and outputs """
 
         def variableColor(variable):
-            if variable.type == 'Real':
-                return QColor.fromRgb(0, 0, 127)
-            elif variable.type in ['Integer', 'Enumeration']:
-                return QColor.fromRgb(255, 127, 0)
+            if variable.type.startswith(('Float', 'Real')):
+                return QColor.fromRgb(26, 77, 179)
+            elif variable.type.startswith(('Enumeration', 'Int', 'UInt')):
+                return QColor.fromRgb(179, 77, 26)
             elif variable.type == 'Boolean':
                 return QColor.fromRgb(255, 0, 255)
             elif variable.type == 'String':
-                return QColor.fromRgb(0, 128, 0)
+                return QColor.fromRgb(26, 114, 16)
+            elif variable.type == 'Binary':
+                return QColor.fromRgb(81, 81, 81)
             else:
                 return QColor.fromRgb(0, 0, 0)
 
@@ -1092,7 +1110,7 @@ class MainWindow(QMainWindow):
         h = max(50., 10 + lh * max(len(inputVariables), len(outputVariables)))
 
         block = QGraphicsRectItem(0, 0, w, h, group)
-        block.setPen(QColor.fromRgb(0, 0, 255))
+        block.setPen(QColor.fromRgb(0, 0, 0))
 
         pen = QPen()
         pen.setWidthF(1)
@@ -1104,12 +1122,12 @@ class MainWindow(QMainWindow):
         y = floor((h - len(inputVariables) * lh) / 2 - 2)
         for variable in inputVariables:
             text = QGraphicsTextItem(variable.name, group)
-            text.setDefaultTextColor(QColor.fromRgb(0, 0, 255))
+            text.setDefaultTextColor(QColor.fromRgb(0, 0, 0))
             text.setFont(font)
             text.setX(3)
             text.setY(y)
 
-            polygon = QPolygonF([QPointF(-13.5, y + 4), QPointF(1, y + 11), QPointF(-13.5, y + 18)])
+            polygon = QPolygonF([QPointF(-8, y + 7.5), QPointF(-1, y + 11), QPointF(-8, y + 14.5)])
 
             path = QPainterPath()
             path.addPolygon(polygon)
@@ -1117,6 +1135,10 @@ class MainWindow(QMainWindow):
             contour = QGraphicsPathItem(path, group)
             contour.setPen(QPen(Qt.NoPen))
             contour.setBrush(variableColor(variable))
+            pen = QPen()
+            pen.setColor(variableColor(variable))
+            pen.setJoinStyle(Qt.MiterJoin)
+            contour.setPen(pen)
 
             y += lh
 
@@ -1124,17 +1146,19 @@ class MainWindow(QMainWindow):
         y = floor((h - len(outputVariables) * lh) / 2 - 2)
         for variable in outputVariables:
             text = QGraphicsTextItem(variable.name, group)
-            text.setDefaultTextColor(QColor.fromRgb(0, 0, 255))
+            text.setDefaultTextColor(QColor.fromRgb(0, 0, 0))
             text.setFont(font)
             text.setX(w - 3 - text.boundingRect().width())
             text.setY(y)
 
-            polygon = QPolygonF([QPointF(w, y + 0 + 7.5), QPointF(w + 7, y + 3.5 + 7.5), QPointF(w, y + 7 + 7.5)])
+            polygon = QPolygonF([QPointF(w + 1, y + 7.5), QPointF(w + 8, y + 11), QPointF(w + 1, y + 14.5)])
 
             path = QPainterPath()
             path.addPolygon(polygon)
             path.closeSubpath()
             contour = QGraphicsPathItem(path, group)
+            contour.setPen(QPen(Qt.NoPen))
+            contour.setBrush(variableColor(variable))
             pen = QPen()
             pen.setColor(variableColor(variable))
             pen.setJoinStyle(Qt.MiterJoin)
@@ -1181,27 +1205,28 @@ class MainWindow(QMainWindow):
                 self.startValues.clear()
                 self.startValues.update(start_values)
 
-    def compilePlatformBinary(self):
+    def compilePlatformBinary(self, target_platform):
         """ Compile the platform binary """
 
         from ..util import compile_platform_binary
 
         platforms = supported_platforms(self.filename)
 
-        if fmpy.platform in platforms:
-            button = QMessageBox.question(self, "Platform binary already exists", "The FMU already contains a binary for the current platform. Do you want to compile and overwrite the existing binary?")
+        if target_platform in platforms:
+            button = QMessageBox.question(self, "Platform binary already exists",
+                                          f'The FMU already contains a binary for the platform "{target_platform}".'
+                                          ' Do you want to compile and overwrite the existing binary?')
             if button == QMessageBox.No:
                 return
 
         try:
-            compile_platform_binary(self.filename)
+            compile_platform_binary(self.filename, target_platform=target_platform)
         except Exception as e:
             QMessageBox.critical(self, "Failed to compile platform binaries", str(e))
             return
 
         self.load(self.filename)
 
-
     def createJupyterNotebook(self):
         """ Create a Juypyter Notebook to simulate the FMU """
 
@@ -1209,7 +1234,7 @@ class MainWindow(QMainWindow):
 
         filename, ext = os.path.splitext(self.filename)
 
-        filename = QFileDialog.getSaveFileName(
+        filename, _ = QFileDialog.getSaveFileName(
             parent=self,
             directory=filename + '.ipynb',
             filter='Jupyter Notebooks (*.ipynb);;All Files (*)'
@@ -1217,10 +1242,19 @@ class MainWindow(QMainWindow):
 
         if filename:
             try:
-                create_jupyter_notebook(self.filename, filename[0])
+                create_jupyter_notebook(self.filename, filename)
             except Exception as e:
                 QMessageBox.critical(self, "Failed to create Jupyter Notebook", str(e))
+                return
 
+            if QMessageBox.question(self, "Open Jupyter Notebook?", f"Start Jupyter and open {filename}?") == QMessageBox.Yes:
+
+                from subprocess import run, CREATE_NEW_CONSOLE
+
+                try:
+                    run(['jupyter', 'notebook', filename], creationflags=CREATE_NEW_CONSOLE)
+                except Exception as e:
+                    QMessageBox.critical(self, "Failed to start Jupyter", str(e))
 
     def createCMakeProject(self):
         """ Create a CMake project from a C code FMU """
@@ -1235,16 +1269,16 @@ class MainWindow(QMainWindow):
         if project_dir:
             create_cmake_project(self.filename, project_dir)
 
-    def addRemoting(self):
-        """ Add 32-bit remoting binaries to the FMU """
+
+    def addRemotingBinaries(self, host_platform, remote_platform):
 
         from ..util import add_remoting
 
         try:
-            add_remoting(self.filename)
+            add_remoting(self.filename, host_platform, remote_platform)
         except Exception as e:
-            QMessageBox.warning(self, "Failed to add 32-bit Remoting",
-                                "Failed to add 32-bit remoting binaries to %s. %s" % (self.filename, e))
+            QMessageBox.warning(self, "Failed to add Remoting Binaries",
+                                f"Failed to add remoting binaries to {self.filename}. {e}")
 
         self.load(self.filename)
 
